#!/bin/bash
VERSION='2024_5'
INSTALLER_VERSION='3.6'
FILE='install.zip'
DRONE_ENGAGE_FOLDER_NAME='drone_engage'
DRONE_COMMUNICATIOR_FOLDER_NAME='de_comm'
DRONE_MAVLINK_FOLDER_NAME='de_mavlink'
DRONE_COMMUNICATIOR_BIN_NAME='de_comm.so'
DRONE_MAVLINK_BIN_NAME='de_ardupilot.so'
DRONE_COMMUNICATIOR_CONFIG_NAME='de_comm.config.module.json'
DRONE_MAVLINK_CONFIG_NAME='de_mavlink.config.module.json'

RED='\033[1;31m'
GREEN='\033[1;32m'
YELLOW='\033[1;33m'
BLUE='\033[1;34m'
NC='\033[0m' # No Color
TMP_OUTPUT=$(mktemp -d -u)
echo -e $GREEN "* DroneEngage Intaller version $INSTALLER_VERSION."  $NC


###################################### Install jq if not exist
if ! command -v jq &> /dev/null
then
    sudo apt install  jq
    sudo rm -rf $TMP_OUTPUT
    echo -e $RED "Please rerun the script" $NC
    exit
fi

###################################### Install systemd for BookWorm
# Read the contents of /etc/os-release into variables
source /etc/os-release
OS=0
# Check if the variable $VERSION_CODENAME contains "buster"
if [[ "$VERSION_CODENAME" == "buster" ]]; then
  echo  -e $GREEN "Debian Buster (10)"  $NC
  OS=1
elif [[ "$VERSION_CODENAME" == "bullseye" ]]; then
  echo  -e $GREEN "Debian Bullseye (11)"  $NC
  OS=2
elif [[ "$VERSION_CODENAME" == "bookworm" ]]; then
  echo  -e $GREEN "Debian BookWorm"  $NC
  sudo apt-get update
  sudo apt-get install systemd


    file_path="/usr/local/openssl/lib/libcrypto.so.1.1"

    if [ -f "$file_path" ]; then
        echo -e $GREEN "File $file_path found." $NC
        export LD_LIBRARY_PATH=/usr/local/openssl/lib:$LD_LIBRARY_PATH
    else
        echo -e $RED "File $file_path does not exist." $NC
        echo -e $YELLOW "INSTALL IT USING SCRIPT:  https://github.com/DroneEngage/DroneEngage_ScriptWiki/blob/main/helper_scripts/install_libcrypto_1.1.sh" $NC
    fi

    

  OS=3
else
  echo  -e $RED "Unknown Debian version"  $NC
  OS=0
fi


##############################################################################
# Unzip Install Files
echo -e $GREEN "* Unzip file $FILE in $TMP_OUTPUT" $NC
echo -e $GREEN "--------------------------------------------" $NC
unzip -qq -o  $FILE  -d $TMP_OUTPUT 

pushd $TMP_OUTPUT


##############################################################################
# Binary Installation

echo -e $GREEN "* Copying Binaries & Checking Binaries Version" $NC

DESTINATION_FOLDER="$HOME/$DRONE_ENGAGE_FOLDER_NAME"
mkdir -p $DESTINATION_FOLDER

### Communicator Binary

####### create folder
mkdir -p $DESTINATION_FOLDER/de_comm
mkdir -p $DESTINATION_FOLDER/de_comm/media
####### backup current file
if [ -f "$DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME" ]; then
    echo -e $YELLOW "Backup current binaries... $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME.bak"  $NC
    mv $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME       $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME.bak
fi
####### copy new file
chmod +x ./drone_engage/de_comm/de_comm.so
cp ./drone_engage/de_comm/de_comm.so $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME
####### run binary --version and display the version
echo -e $BLUE "-----running $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME -v" $NC
$DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_BIN_NAME -v

######
######
######
######

### Mavlink Binary
####### create folder
mkdir -p $DESTINATION_FOLDER/de_mavlink
####### backup current file
if [ -f "$DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME" ]; then
    echo -e $YELLOW "Backup current binaries... $DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME.bak"  $NC
    mv $DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME       $DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME.bak
fi
####### copy new file
chmod +x ./drone_engage/de_mavlink/de_ardupilot.so
cp ./drone_engage/de_mavlink/de_ardupilot.so $DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME
####### run binary --version and display the version
echo -e $BLUE "-----running $DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME -v" $NC
$DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_BIN_NAME -v

##
##############################################################################

##############################################################################
# Config Files Installation
echo ""
echo ""
echo -e $GREEN "* Copying Configuration Files" $NC

## Reads a json param from a json file
read_param () {
	local file_name=$1
    param_name=$2
	
	
    VALUE=$(cat $file_name | sed 's/^\s*/ /' | sed 's/[[:blank:]][[:blank:]]*\/\/.*//' |  sed -n '/ \/\/.*/!p' | sed 's|/\*|\n&|g;s|*/|&\n|g' | sed '/\/\*/,/*\//d' | jq .$param_name)
    ## Normal print
    echo -e  $YELLOW"read " $GREEN"$param_name"$YELLOW":"$GREEN"$VALUE" $NC >&2
    ## Return 
    echo $VALUE
}


## Writes a json param to json file
write_param () {
	local file_name=$1
    param_name=$2
	param_value=$3
	
    RET=$(cat $file_name  | sed 's/^\s*/ /' | sed 's/[[:blank:]][[:blank:]]*\/\/.*//' |  sed -n '/ \/\/.*/!p' | sed 's|/\*|\n&|g;s|*/|&\n|g' | sed '/\/\*/,/*\//d' | jq ".$param_name=$param_value" > $file_name.$param_name)
    cat $file_name.$param_name > $file_name
    rm $file_name.$param_name
    #echo -e "$RET"
    #$RET
}

write_param2 () {
	local file_name=$1
    param_name=$2
	param_value=$3
	#echo "cat $file_name  | sed 's/^\s*/ /' | sed 's/[[:blank:]][[:blank:]]*\/\/.*//' |  sed -n '/ \/\/.*/!p' | sed 's|/\*|\n&|g;s|*/|&\n|g' | sed '/\/\*/,/*\//d' | jq ".$param_name=\"$param_value\"" > $file_name.$param_name"
    RET=$(cat $file_name  | sed 's/^\s*/ /' | sed 's/[[:blank:]][[:blank:]]*\/\/.*//' |  sed -n '/ \/\/.*/!p' | sed 's|/\*|\n&|g;s|*/|&\n|g' | sed '/\/\*/,/*\//d' | jq ".$param_name=\"$param_value\"" > $file_name.$param_name)
    cat $file_name.$param_name > $file_name
    rm $file_name.$param_name
}

write_if_exists ()
{
    local file_name=$1
    read_param=$2
    VALUE=$(read_param "$file_name.bak" "$read_param")
    if [[ $VALUE == "null" ]]; then
            echo "Field  $read_param does not exist in file $file_name"
    else
        echo "Read : $VALUE"
        write_param "$file_name" "$read_param" "$VALUE"
    fi
}

## ASK for parameter
ask_param () {
    title=$1
    question=$2
    default_value=$3

    RES=$(whiptail --inputbox "$question" 8 39 $default_value --title "$title" 3>&1 1>&2 2>&3)
                                                                        # A trick to swap stdout and stderr.
    # Again, you can pack this inside if, but it seems really long for some 80-col terminal users.
    exitstatus=$?
    if [ $exitstatus = 0 ]; then
        echo $RES
        return 
    else
        echo $default_value
        return 
    fi

    echo $default_value
}

copy_and_ask_param()
{
    local file_name=$1
    param_name=$2
    title=$3
    is_num=$4

    VALUE=$(read_param "$file_name" "$param_name")
    VALUE="${VALUE//\"}"  #remove ""
    VALUE=$(ask_param "$param_name" "$title" $VALUE)
    if [[ $is_num -eq 1 ]]; then
    write_param "$file_name" "$param_name" "$VALUE"
    else
    write_param2 "$file_name" "$param_name" "$VALUE"
    fi
}


#### Backup Comm Config File
FULL_COMM_CONFIG_PATH=$(echo "$DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/$DRONE_COMMUNICATIOR_CONFIG_NAME")
if [ -f $FULL_COMM_CONFIG_PATH ]; then
    echo -e $YELLOW "Backup current config... $FULL_COMM_CONFIG_PATH.bak"  $NC
    mv $FULL_COMM_CONFIG_PATH       $FULL_COMM_CONFIG_PATH.bak
else
    echo -e $GREEN "Creating configuration file at:"  $YELLOW  $FULL_COMM_CONFIG_PATH $NC

fi
cp ./drone_engage/de_comm/$DRONE_COMMUNICATIOR_CONFIG_NAME $FULL_COMM_CONFIG_PATH
cp ./drone_engage/de_comm/root.crt $DESTINATION_FOLDER/$DRONE_COMMUNICATIOR_FOLDER_NAME/root.crt

if [ -f $FULL_COMM_CONFIG_PATH.bak ]; then
echo -e $GREEN "Previous settings are backed in $FULL_COMM_CONFIG_PATH.bak" $NC
#Copy Parameters
write_if_exists "$FULL_COMM_CONFIG_PATH" "module_id"
write_if_exists "$FULL_COMM_CONFIG_PATH" "s2s_udp_listening_ip"
write_if_exists "$FULL_COMM_CONFIG_PATH" "s2s_udp_listening_port"
write_if_exists "$FULL_COMM_CONFIG_PATH" "s2s_udp_packet_size"
write_if_exists "$FULL_COMM_CONFIG_PATH" "auth_ip"
write_if_exists "$FULL_COMM_CONFIG_PATH" "auth_port"
write_if_exists "$FULL_COMM_CONFIG_PATH" "auth_verify_ssl"
write_if_exists "$FULL_COMM_CONFIG_PATH" "root_certificate_path"
write_if_exists "$FULL_COMM_CONFIG_PATH" "userName"
write_if_exists "$FULL_COMM_CONFIG_PATH" "accessCode"
write_if_exists "$FULL_COMM_CONFIG_PATH" "unitID"
write_if_exists "$FULL_COMM_CONFIG_PATH" "unitDescription"
write_if_exists "$FULL_COMM_CONFIG_PATH" "logger_enabled"
write_if_exists "$FULL_COMM_CONFIG_PATH" "logger_debug"
fi


echo -e $YELLOW
read -p "Do you want to edit parameter values (y/n)? " -n 1 -r
echo -e $NC    # (optional) move to a new line
if [[ $REPLY =~ ^[Yy]$ ]]
then
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "module_id" "Communication Module Name" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "s2s_udp_listening_ip" "Communication Module UDP IP" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "s2s_udp_listening_port" "Communication Module UDP Port" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "auth_ip" "Server IP" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "auth_port" "Server Port" 1
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "userName" "Account Name" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "accessCode" "Account Password" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "unitID" "Vehicle Name" 0
    copy_and_ask_param "$FULL_COMM_CONFIG_PATH" "unitDescription" "Vehicle Description" 0
fi

echo ""
echo "for full edit use:"
echo "nano $FULL_COMM_CONFIG_PATH"
echo ""
echo ""
read -p "press ENTER to continue"
########################################################################
########################################################################

#### Backup mavlink Config File
echo ""
echo ""
FULL_MAVLINK_CONFIG_PATH=$(echo "$DESTINATION_FOLDER/$DRONE_MAVLINK_FOLDER_NAME/$DRONE_MAVLINK_CONFIG_NAME")
if [ -f $FULL_MAVLINK_CONFIG_PATH ]; then
    echo -e $YELLOW "Backup current mavlink config... $FULL_MAVLINK_CONFIG_PATH.bak"  $NC
    mv $FULL_MAVLINK_CONFIG_PATH     $FULL_MAVLINK_CONFIG_PATH.bak
else
    echo -e $GREEN "Creating configuration file at:"  $YELLOW  $FULL_MAVLINK_CONFIG_PATH $NC
fi
cp ./drone_engage/de_mavlink/$DRONE_MAVLINK_CONFIG_NAME $FULL_MAVLINK_CONFIG_PATH

if [ -f $FULL_MAVLINK_CONFIG_PATH.bak ]; then
echo -e $GREEN "Previous settings are backed in $FULL_MAVLINK_CONFIG_PATH.bak" $NC
echo -e $YELLOW
#Copy Parameters
write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "module_id"
write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_target_ip"
write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_target_port"
write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_listening_ip"
write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_listening_port"
write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_packet_size"
write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "fcb_connection_uri"

VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "logger_enabled")
if [ VALUE==null ]; then
VALUE=false
fi
write_param "$FULL_MAVLINK_CONFIG_PATH" "logger_enabled" "$VALUE"

VALUE=$(read_param "$FULL_MAVLINK_CONFIG_PATH.bak" "logger_debug")
if [ VALUE==null ]; then
VALUE=false
fi
write_param "$FULL_MAVLINK_CONFIG_PATH" "logger_debug" "$VALUE"

write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "event_fire_channel"
write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "event_wait_channel"
write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "read_only_mode"
write_if_exists "$FULL_MAVLINK_CONFIG_PATH" "ignore_loading_parameters"
fi

echo -e $YELLOW
read -p "Do you want to edit parameter values (y/n)? " 
echo -e $NC    # (optional) move to a new line
if [[ $REPLY =~ ^[Yy]$ ]]
then
    copy_and_ask_param "$FULL_MAVLINK_CONFIG_PATH" "module_id" "Mavlink Module Name" 0
    copy_and_ask_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_target_ip" "Communication Module UDP IP" 0
    copy_and_ask_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_target_port" "Communication Module UDP Port" 0
    copy_and_ask_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_listening_ip" "Mavlink Module UDP IP" 0
    copy_and_ask_param "$FULL_MAVLINK_CONFIG_PATH" "s2s_udp_listening_port" "Mavlink Module UDP Port" 0
fi


echo ""
echo "for full edit use:"
echo "nano $FULL_MAVLINK_CONFIG_PATH"
echo ""
echo ""
read -p "press ENTER to continue"

##
##############################################################################


##############################################################################
# Service Installation
service_exists() {
    local n=$1
    if [[ $(systemctl list-units --all -t service --full --no-legend "$n.service" | sed 's/^\s*//g' | cut -f1 -d' ') == $n.service ]]; then
        return 0
    else
        return 1
    fi
}

echo -e $GREEN "--------------------------------------------" $NC

if service_exists de_communicator; then
    echo -e $YELLOW "stopping service de_communicator.service" $NC
    sudo systemctl stop de_communicator.service 
fi

if service_exists de_mavlink; then
    echo -e $YELLOW "stopping service de_mavlink.service" $NC
    sudo systemctl stop de_mavlink.service 
fi



# Perform the find and replace operation using sed
# change file path from /home/pi to whatever $HOME
sed -i "s|/home/pi|$HOME|g" "./de_communicator.service"
sed -i "s|/home/pi|$HOME|g" "./de_mavlink.service"

# In Debian BookWorm we need to edit service files to add lib path
if [ "$OS" -eq 3 ]; then
    # Set the environment variable line to be added
    environment_line=" Environment=\"LD_LIBRARY_PATH=/usr/local/openssl/lib:\$LD_LIBRARY_PATH\""

    # Find the line number of [Service] and add the environment line after it
    line_number=$(awk '/\[Service\]/{ print NR; exit }' "./de_communicator.service")
    if [ -n "$line_number" ]; then
    sed -i "${line_number}a $environment_line" "./de_communicator.service"
    echo "Environment line added to $service_file"
    else
    echo "[Service] section not found in ./de_communicator.service"
    fi
fi

sudo mv ./de_communicator.service  /lib/systemd/system/de_communicator.service
sudo mv ./de_mavlink.service  /lib/systemd/system/de_mavlink.service

sudo chmod +x ./drone_engage/de_comm/de_comm.so
sudo chmod +x ./drone_engage/de_mavlink/de_ardupilot.so

echo -e $GREEN "Enabling de_communicator.service" $NC
sudo systemctl enable de_communicator.service 
echo -e $GREEN "Enabling de_mavlink.service" $NC
sudo systemctl enable de_mavlink.service
sudo systemctl daemon-reload 
sudo systemctl restart de_communicator.service 
sudo systemctl restart de_mavlink.service



if [ $(systemctl is-active de_communicator.service) == "active" ] ; then
    echo -e $GREEN "service de_communicator.service is active" $NC
else
    echo -e $RED "WARNING: " $YELLOW "service de_communicator.service is not active" $NC
fi

if [ $(systemctl is-active de_mavlink.service) == "active" ] ; then
    echo -e $GREEN "service de_mavlink.service is active" $NC
else
    echo -e $RED "WARNING: " $YELLOW "service de_mavlink.service is not active" $NC
fi
##
##############################################################################

sudo rm -rf $TMP_OUTPUT

echo "DONE"        

